package sun.net.www;

import sun.net.www.http.ChunkedInputStream;
import java.io.IOException;
import java.io.InputStream;
import sun.net.ProgressSource;
import java.io.FilterInputStream;

public class MeteredStream extends FilterInputStream
{
    protected boolean closed;
    protected long expected;
    protected long count;
    protected long markedCount;
    protected int markLimit;
    protected ProgressSource pi;
    
    public MeteredStream(InputStream in, ProgressSource pi, long expected) {
        super(in);
        this.closed = false;
        this.count = 0L;
        this.markedCount = 0L;
        this.markLimit = -1;
        this.pi = pi;
        this.expected = expected;
        if (pi != null) {
            pi.updateProgress(0L, expected);
        }
    }
    
    private final void justRead(long n) throws IOException {
        if (n == -1L) {
            if (!this.isMarked()) {
                this.close();
            }
            return;
        }
        else {
            this.count += n;
            if (this.count - this.markedCount > (long)this.markLimit) {
                this.markLimit = -1;
            }
            if (this.pi != null) {
                this.pi.updateProgress(this.count, this.expected);
            }
            if (this.isMarked()) {
                return;
            }
            if (this.expected > 0L && this.count >= this.expected) {
                this.close();
            }
            return;
        }
    }
    
    private boolean isMarked() {
        return this.markLimit >= 0 && this.count - this.markedCount <= (long)this.markLimit;
    }
    
    @Override
    public synchronized int read() throws IOException {
        if (this.closed) {
            return -1;
        }
        int read = this.in.read();
        if (read != -1) {
            this.justRead(1L);
        }
        else {
            this.justRead((long)read);
        }
        return read;
    }
    
    @Override
    public synchronized int read(byte[] b, int off, int len) throws IOException {
        if (this.closed) {
            return -1;
        }
        int read = this.in.read(b, off, len);
        this.justRead((long)read);
        return read;
    }
    
    @Override
    public synchronized long skip(long n) throws IOException {
        if (this.closed) {
            return 0L;
        }
        if ((this.in instanceof ChunkedInputStream)) {
            n = this.in.skip(n);
        }
        else {
            n = ((n > this.expected - this.count) ? (this.expected - this.count) : n);
            n = this.in.skip(n);
        }
        this.justRead(n);
        return n;
    }
    
    @Override
    public void close() throws IOException {
        if (this.closed) {
            return;
        }
        if (this.pi != null) {
            this.pi.finishTracking();
        }
        this.closed = true;
        this.in.close();
        return;
    }
    
    @Override
    public synchronized int available() throws IOException {
        return this.closed ? 0 : this.in.available();
    }
    
    @Override
    public synchronized void mark(int n) {
        if (this.closed) {
            return;
        }
        super.mark(n);
        this.markedCount = this.count;
        this.markLimit = n;
        return;
    }
    
    @Override
    public synchronized void reset() throws IOException {
        if (this.closed) {
            return;
        }
        if (!this.isMarked()) {
            throw new IOException("Resetting to an invalid mark");
        }
        this.count = this.markedCount;
        super.reset();
        return;
    }
    
    @Override
    public boolean markSupported() {
        return !this.closed && super.markSupported();
    }
    
    @Override
    protected void finalize() throws Throwable {
        try {
            this.close();
            if (this.pi != null) {
                this.pi.close();
            }
        }
        finally {
            super.finalize();
        }
    }
}
